@@ -55,7 +55,7 @@ public:
 	 * ---------------------------------------------
 	 */
 	void validate() override;
-	void evaluate(std::vector<double> &image) override;
+	void evaluate(Image &image, const Mask &mask, const PixelScale &scale, double magzero) override;
 
 protected:
 
