@@ -149,7 +149,7 @@ void RadialProfile::initial_calculations() {
 	double box = this->box + 2;
 	double r_box = M_PI * box / (2*beta(1/box, 1/box));
 	double lumtot = this->get_lumtot(r_box);
-	this->_ie = std::pow(10, -0.4*(this->mag - this->model.magzero))/lumtot;
+	this->_ie = std::pow(10, -0.4*(this->mag - magzero))/lumtot;
 
 	/*
 	 * Optionally adjust the user-given rscale_switch and resolution parameters
@@ -216,8 +216,8 @@ void RadialProfile::validate() {
 /**
  * The scale by which each image pixel value is multiplied
  */
-double RadialProfile::get_pixel_scale() {
-	double pixel_area = this->model.scale_x * this->model.scale_y;
+double RadialProfile::get_pixel_scale(const PixelScale &scale) {
+	double pixel_area = scale.first * scale.second;
 	return pixel_area * this->_ie;
 }
 
@@ -231,7 +231,9 @@ void RadialProfile::subsampling_params(double x, double y,
 /**
  * The main profile evaluation function
  */
-void RadialProfile::evaluate(std::vector<double> &image) {
+void RadialProfile::evaluate(Image &image, const Mask &mask, const PixelScale &scale, double magzero) {
+
+	this->magzero = magzero;
 
 	/*
 	 * Perform all the pre-calculations needed by the radial profiles
@@ -248,24 +250,24 @@ void RadialProfile::evaluate(std::vector<double> &image) {
 #endif /* PROFIT_DEBUG */
 
 #ifndef PROFIT_OPENCL
-	evaluate_cpu(image);
+	evaluate_cpu(image, mask, scale);
 #else
 	/*
 	 * We fallback to the CPU implementation if no OpenCL context has been
 	 * given, or if there is no OpenCL kernel implementing the profile
 	 */
-	auto env = model.opencl_env;
-	if( !env || !supports_opencl() ) {
-		evaluate_cpu(image);
+	auto env = OpenCLEnvImpl::fromOpenCLEnvPtr(model.get_opencl_env());
+	if( force_cpu || !env || !supports_opencl() ) {
+		evaluate_cpu(image, mask, scale);
 		return;
 	}
 
 	try {
 		if( env->use_double ) {
-			evaluate_opencl<double>(image);
+			evaluate_opencl<double>(image, mask, scale, env);
 		}
 		else {
-			evaluate_opencl<float>(image);
+			evaluate_opencl<float>(image, mask, scale, env);
 		}
 	} catch (const cl::Error &e) {
 		std::ostringstream os;
@@ -276,12 +278,14 @@ void RadialProfile::evaluate(std::vector<double> &image) {
 
 }
 
-void RadialProfile::evaluate_cpu(std::vector<double> &image) {
+void RadialProfile::evaluate_cpu(Image &image, const Mask &mask, const PixelScale &scale) {
 
-	double half_xbin = model.scale_x/2.;
-	double half_ybin = model.scale_y/2.;
+	double half_xbin = scale.first/2.;
+	double half_ybin = scale.second/2.;
 
-	double scale = this->get_pixel_scale();
+	auto width = image.getWidth();
+	auto height = image.getHeight();
+	double flux_scale = this->get_pixel_scale(scale);
 
 	/*
 	 * If compiled with OpenMP support, and if the user requests so,
@@ -291,17 +295,17 @@ void RadialProfile::evaluate_cpu(std::vector<double> &image) {
 	bool use_omp = model.omp_threads > 1;
 	#pragma omp parallel for collapse(2) schedule(dynamic, 10) if(use_omp) num_threads(model.omp_threads)
 #endif /* PROFIT_OPENMP */
-	for(unsigned int j=0; j < model.height; j++) {
-		for(unsigned int i=0; i < model.width; i++) {
+	for(unsigned int j=0; j < height; j++) {
+		for(unsigned int i=0; i < width; i++) {
 
 			/* We were instructed to ignore this pixel */
-			if( !model.calcmask.empty() && !model.calcmask[i + j*model.width] ) {
+			if( mask && !mask[i + j * width] ) {
 				continue;
 			}
 
 			double x_prof, y_prof, r_prof;
-			double y = half_ybin + j*model.scale_y;
-			double x = half_xbin + i*model.scale_x;
+			double y = half_ybin + j * scale.second;
+			double x = half_xbin + i * scale.first;
 			this->_image_to_profile_coordinates(x, y, x_prof, y_prof);
 
 			/*
@@ -328,7 +332,7 @@ void RadialProfile::evaluate_cpu(std::vector<double> &image) {
 				                                   0, max_recursions, resolution);
 			}
 
-			image[i + j*model.width] = scale * pixel_val;
+			image[i + j * width] = flux_scale * pixel_val;
 		}
 	}
 
@@ -427,7 +431,7 @@ std::chrono::nanoseconds::rep to_nsecs(const std::chrono::system_clock::duration
 }
 
 template <typename FT>
-void RadialProfile::evaluate_opencl(std::vector<double> &image) {
+void RadialProfile::evaluate_opencl(Image &image, const Mask &mask, const PixelScale &scale, OpenCLEnvImplPtr &env) {
 
 #define AS_FT(x) static_cast<FT>(x)
 
@@ -436,9 +440,8 @@ void RadialProfile::evaluate_opencl(std::vector<double> &image) {
 	typedef ss_info_t<FT> ss_info_t;
 	typedef ss_kinfo_t<FT> ss_kinfo_t;
 
-	unsigned int imsize = model.width * model.height;
+	unsigned int imsize = image.size();
 
-	auto env = model.opencl_env;
 	OpenCL_times cl_times0, ss_cl_times;
 	RadialProfileStats* stats = static_cast<RadialProfileStats *>(this->stats.get());
 
@@ -454,11 +457,11 @@ void RadialProfile::evaluate_opencl(std::vector<double> &image) {
 	cl::Kernel kernel = env->get_kernel(kname);
 	kernel.setArg(arg++, image_buffer);
 	kernel.setArg(arg++, subsampling_points_buffer);
-	kernel.setArg(arg++, model.width);
-	kernel.setArg(arg++, model.height);
+	kernel.setArg(arg++, image.getWidth());
+	kernel.setArg(arg++, image.getHeight());
 	kernel.setArg(arg++, (int)rough);
-	kernel.setArg(arg++, AS_FT(model.scale_x));
-	kernel.setArg(arg++, AS_FT(model.scale_y));
+	kernel.setArg(arg++, AS_FT(scale.first));
+	kernel.setArg(arg++, AS_FT(scale.second));
 	add_common_kernel_parameters<FT>(arg, kernel);
 	t_kprep = system_clock::now();
 
@@ -535,7 +538,7 @@ void RadialProfile::evaluate_opencl(std::vector<double> &image) {
 		unsigned int resolution, max_recursions;
 		subsampling_params(point.x, point.y, resolution, max_recursions);
 		top_recursions = std::max(top_recursions, max_recursions);
-		last_ss_info.push_back({point, AS_FT(model.scale_x), AS_FT(model.scale_y), resolution, max_recursions});
+		last_ss_info.push_back({point, AS_FT(scale.first), AS_FT(scale.second), resolution, max_recursions});
 	}
 
 	auto ss_kname = name + "_subsample_" + float_traits<FT>::name;
@@ -665,17 +668,17 @@ void RadialProfile::evaluate_opencl(std::vector<double> &image) {
 
 	t_loopend = system_clock::now();
 
-	std::for_each(subimages_results.begin(), subimages_results.end(), [&image, this](const im_result_t &res) {
-		FT x = res.point.x / model.scale_x;
-		FT y = res.point.y / model.scale_y;
-		unsigned int idx = static_cast<unsigned int>(floor(x)) + static_cast<unsigned int>(floor(y)) * model.width;
+	std::for_each(subimages_results.begin(), subimages_results.end(), [&image, &scale](const im_result_t &res) {
+		FT x = res.point.x / scale.first;
+		FT y = res.point.y / scale.second;
+		unsigned int idx = static_cast<unsigned int>(floor(x)) + static_cast<unsigned int>(floor(y)) * image.getWidth();
 		image[idx] += res.value;
 	});
 
 	// the image needs to be multiplied by the pixel scale
-	double scale = this->get_pixel_scale();
-	std::transform(image.begin(), image.end(), image.begin(), [scale](double pixel) {
-		return pixel * scale;
+	double flux_scale = this->get_pixel_scale(scale);
+	std::transform(image.begin(), image.end(), image.begin(), [flux_scale](double pixel) {
+		return pixel * flux_scale;
 	});
 	t_imgtrans = system_clock::now();
 
@@ -725,8 +728,10 @@ RadialProfile::RadialProfile(const Model &model, const std::string &name) :
 	rscale_switch(1), resolution(9),
 	max_recursions(2), adjust(true),
 	rscale_max(0),
+	force_cpu(false),
 	rscale(0), _ie(0),
-	_cos_ang(0), _sin_ang(0)
+	_cos_ang(0), _sin_ang(0),
+	magzero(0)
 {
 	// no-op
 }
@@ -745,6 +750,7 @@ bool RadialProfile::parameter_impl(const std::string &name, bool value) {
 
 	if( name == "rough" )              { rough = value; }
 	else if( name == "adjust" )        { adjust = value; }
+	else if( name == "force_cpu" )     { force_cpu = value; }
 	else {
 		return false;
 	}
